﻿using System;
using System.Windows;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Controls;
using Telerik.Windows.Controls;

namespace SampleBinding_SL
{
	public partial class MainPage : UserControl
	{
		private MainViewModel mainVM;

		public MainPage()
		{
			InitializeComponent();

			this.mainVM = new MainViewModel();
			this.xTreeView.ItemsSource = this.mainVM;
			this.xTreeView.AddHandler(KeyDownEvent, new System.Windows.Input.KeyEventHandler(this.xTreeView_KeyDown), true);
		}

		private bool invalidInputCommitAttempt = false;
        
        private bool isAlertOpen = false;
        private RadTreeViewItem editedItem;

		private void xTreeView_PreviewEdited(object sender, Telerik.Windows.Controls.RadTreeViewItemEditedEventArgs e)
		{
			var data = ((SampleBinding_SL.BusinessItem)(e.NewValue));

			if (data.Name.Equals("error"))
			{
                e.Handled = true;
                if (!this.isAlertOpen)
                {                   
                    RadWindow.Alert(new DialogParameters()
                    {
                        Content = "Error!",
                        Closed = MainPage_Closed
                    });

                    this.isAlertOpen = true;
                    this.editedItem = e.OriginalSource as RadTreeViewItem;
                }
                else
                {
                    this.isAlertOpen = false;
                }

				this.invalidInputCommitAttempt = true;
			}
			else
			{
				this.invalidInputCommitAttempt = false;
			}
		}

        private void MainPage_Closed(object sender, WindowClosedEventArgs e)
        {
            TextBox editBox = this.editedItem.FindChildByType<TextBox>();
            editBox.Focus();
            this.editedItem = null;   
        }

    
		private void xTreeView_PreviewEditCanceled(object sender, RadTreeViewItemEditedEventArgs e)
		{
			if (this.invalidInputCommitAttempt)
			{
				//this.invalidInputCommitAttempt = false;
				e.Handled = true;
			}
		}

		private void xTreeView_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
		{
			var editedItem = (e.OriginalSource as FrameworkElement).ParentOfType<RadTreeViewItem>();

			//if (editedItem != null && editedItem.IsInEditMode && e.Key == System.Windows.Input.Key.Tab)
			if (this.invalidInputCommitAttempt && e.Key == System.Windows.Input.Key.Tab)
			{
				e.Handled = true;
			}


			if (e.Key == System.Windows.Input.Key.Escape)
			{
				this.invalidInputCommitAttempt = false;
				if (editedItem != null)
				{
					editedItem.IsInEditMode = false;
				}
				e.Handled = true;
			}
		}
	}

	public class MainViewModel : ObservableCollection<BusinessItem>
	{
		public MainViewModel()
		{
			for (int i = 0; i < 100; i++)
			{
				BusinessItem bi = new BusinessItem()
				{
					Name = i.ToString(),
					Level = 1,
				};

				for (int j = 0; j < 5; j++)
				{
					BusinessItem bi2 = new BusinessItem()
					{
						Name = bi.Name + "." + j,
						Level = 2,
					};

					for (int k = 0; k < 5; k++)
					{
						BusinessItem bi3 = new BusinessItem()
						{
							Name = bi2.Name + "." + k,
							Level = 3,
						};
						bi2.Items.Add(bi3);
					}

					bi.Items.Add(bi2);
				}

				this.Add(bi);
			}
		}
	}

	public class BusinessItem : INotifyPropertyChanged
	{
		public event PropertyChangedEventHandler PropertyChanged;

		public override string ToString()
		{
			return this.Name + " @ " + this.Level;
		}

		public BusinessItem()
		{
			this.Items = new ObservableCollection<BusinessItem>();
		}
		public ObservableCollection<BusinessItem> Items { get; set; }
		public string name;
		public string Name
		{
			get
			{
				return this.name;
			}
			set
			{
				if (this.name != value)
				{
					this.name = value;
					this.OnPorpertyChanged("Name");
				}
			}
		}
		public int Level { get; set; }

		private void OnPorpertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

	}
}
